;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;; Little handy routines
;;;;; Written by Yip Chi Lap [Beta] <clyip@csis.hku.hk>
;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Written on 24-05-1997[6]1645
;;;
(defun toggle-truncate-lines ()
  "Toggle the value of the variable truncate-lines"
  (interactive)
  (setq truncate-lines (not truncate-lines)))

;;;
;;; Name changed 08-12-1997[1]
;;; 15 August 1996
;;;
(defun untabify-buffer ()
  "Untabify the whole buffer without moving point and mark."
  (interactive)
  (untabify (point-min) (point-max)))

;;;
;;; Delete horizontal space at the end of the line within region
;;;
;;; 15-12-1998[2] First version
;;;
(defun delete-space-eol-region (start end)
  "Delete whitespace at the end of lines within region"
  (interactive "r")
  (save-excursion
    (save-restriction
      (narrow-to-region (point-min) end)
      (goto-char start)
      (while (re-search-forward "[ \t]+$" nil t)
	(replace-match "" nil nil)))))

;;;
;;; Delete whitespace at the end of lines in the buffer
;;;
;;; 15-12-1998[2] First version
;;;
(defun delete-space-eol-buffer ()
  "Delete whitespace at the end of lines in the buffer"
  (interactive)
  (delete-space-eol-region (point-min) (point-max)))

;;;
;;; Convert hexadecimal string to decimal
;;; Return the converted number of nil if there is error during conversion
;;; String of length 0 is converted into 0
;;; Written on 07-11-1996[5]
;;;
(defun hex-to-dec (hex-str)
  "Convert hexadecimal string to decimal"
  (let ((retval 0)
	(r 0)
	(endpos (length hex-str)))
    (catch 'invalid
      (while (< r endpos)
	(let ((ch (elt hex-str r)))
	  (cond ((and (>= ch 48) (<= ch 57)) 
		  (setq retval (logior (lsh retval 4) (- ch 48))))
		((and (>= ch 97) (<= ch 102)) 
		  (setq retval (logior (lsh retval 4) (- ch 87))))
		((and (>= ch 65) (<= ch 70)) 
		  (setq retval (logior (lsh retval 4) (- ch 55))))
		(t (throw 'invalid nil))))
	(setq r (1+ r)))
      retval)))

;;;
;;; Template for a C++ program
;;; Changes values of point and mark.
;;;
;;; Written sometime on or before 10-1996
;;; Modified:
;;;   28-11-1996[4]1056   Use only one insert command
;;;   22-04-1997[2]1749   Added long === lines, argc and argv
;;;   23-02-1999[2]1430   Used ### lines instead of === lines
;;;
(defun c++-program ()
  "C++ program template"
  (interactive)
  (insert 
"// -*- C++ -*-==============================================================\n"
"//\n"
"// \n"
"//\n"
"//==========================================================================\n"
"#include <iostream>\n"
"//##########################################################################\n"
"//==========================================================================\n"
"//--------------------------------------------------------------------------\n"
"int main(int argc, char **argv, char **envp)\n{\n  \n  return 0;\n}\n"
"//##########################################################################\n")
  (end-of-line -3)
  (set-mark (point))
  (end-of-line -8))

;;;
;;; for the usage function
;;;
(defun c++-usage ()
  "C++ usage template"
  (interactive)
  (insert 
"
int usage(const char *progpath)
{
  const char *progname=progpath;
  // find basename
  while (*progpath)
    if (*progpath++=='/')
      progname=progpath;
  // print message
  cout<<progname<<\": WHATEVER\\n\"
        \"Usage: \"<<progname<<\" [Options]\\n\"
        \"Options:\\n\"
        \"  -x   WHATEVER\"
      <<endl;
  return 1;
}
"))

;;;
;;; Returns an ifdef symbol from a filename
;;; Converts all . to _ and prefix it by a _
;;;
;;; Written
;;;   18-06-1998[4]1735   First version
;;;
(defun filename-to-ifdef-symbol (s)
 (let ((ss (copy-sequence s))
       (idx (length s)))
   (while (> idx 0)
    (setq idx (1- idx))
    (if (= (aref ss idx) ?.) (aset ss idx ?_)))
  (concat "_" ss)))

;;;
;;; Written
;;;   15-12-1998[2]0949   Added comment section and used (end-of-line -9)
;;;   19-06-1998[5]1005   Changed the interactive interface
;;;   27-03-1997[4]1056   Fancier output
;;;   28-11-1996[4]1056   First version
;;;
(defun c++-header ()
  (interactive)
  "C++ header template"
  (let ((name (filename-to-ifdef-symbol
	       (file-name-nondirectory (buffer-file-name)))))
    (insert 
"// -*- C++ -*-==============================================================\n"
"//\n"
"// \n"
"//\n"
"//==========================================================================\n"
"#ifndef " name "\n"
"#define " name "\n"
"//##########################################################################\n"
"//==========================================================================\n"
"//--------------------------------------------------------------------------\n"
"//##########################################################################\n"
"#endif  // " name "\n"
"//##########################################################################\n")
  (end-of-line -10)
  (c++-mode)))

;;;
;;; Written
;;;   28-11-1996[4]1056   Fancier output
;;;   27-03-1996[4]1056   First version
;;;
(defun c++-fn ()
  "C++ function file template"
  (interactive)
  (let ((name (concat (file-name-sans-extension
		       (file-name-nondirectory (buffer-file-name)))
		      ".h")))
  (insert
"// -*- C++ -*-==============================================================\n"
"//\n"
"// \n"
"//\n"
"//==========================================================================\n"
"#include \"" name "\"\n"
"//##########################################################################\n"
"//==========================================================================\n"
"//--------------------------------------------------------------------------\n"
"//##########################################################################\n")
  (end-of-line -7)
  (c++-mode)))

;;;
;;; Written
;;;   05-06-1997[4]  First version
;;;
(defun c++-class ()
  "C++ class template"
  (interactive)
  (insert "class \n{\npublic:\nprotected:\nprivate:\n};")
  (previous-line 5)
  (end-of-line))
